//
//  main.swift
//  URL Handling - Swift
//
//  Created by Sam Marshall on 6/4/14.
//
//

// Usage: ./url-handling-swift [url]

import Foundation

if (C_ARGC == 2) {
	
	var input_url = NSString.stringWithCString(C_ARGV[1]) as NSString;
	
	println("Input: \(input_url)");
	
	var hasHTTP = input_url.hasPrefix("http://") as Bool;
	var hasHTTPS = input_url.hasPrefix("https://") as Bool;
	var hasWWW = input_url.hasPrefix("www") as Bool;
	
	if (hasHTTP || hasHTTPS || hasWWW) {
		
		println("This appears to be a valid url.");
		
		var prefix_array:Array<CString> = [];
		
		if (hasHTTP) {
			input_url = input_url.stringByReplacingOccurrencesOfString("http://", withString: "");
			prefix_array.append("http://");
		}
		
		if (hasHTTPS) {
			input_url = input_url.stringByReplacingOccurrencesOfString("https://", withString: "");
			prefix_array.append("https://");
		}
		
		if (hasWWW) {
			input_url = input_url.stringByReplacingOccurrencesOfString("www.", withString: "");
			prefix_array.append("www.");
		}
		
		var url_components = input_url.componentsSeparatedByString("/") as NSArray;
		
		var has_base_domain_length = url_components[0].length();
		
		var output_string = NSMutableString.stringWithString(url_components.componentsJoinedByString("/")) as NSMutableString;
		
		if (has_base_domain_length > 0) {
			for item in prefix_array {
				var item_string = NSString.stringWithCString(item) as NSString;
				output_string.insertString(item_string, atIndex: 0);
			}
		}
		println("Output: \(output_string)");
		
		var open_nsstring = NSString.stringWithString("open \(output_string)");
		var opening = open_nsstring.cStringUsingEncoding(NSUTF8StringEncoding);
		
		/*
		 This is using a call to system() to take a shortcut in replicating the "openURL" functionality provided
			by the custom core library used in the original software. The real code was much more complex but resulted
			in the same resulting call as it was expecting valid web URLs and registed URL-schemes on the host 
			(e.g.: http://, https://, rdar://, feed://, irc://, etc)
		*/
		system(opening);
	}
	else {
		println("Invalid URL.");
	}
}
else {
	println("Please supply a URL.");
}
